'use strict';

var umiSerializers = require('@metaplex-foundation/umi-serializers');

// src/lamports.ts
var maxU64Value = 18446744073709551615n;
function isLamports(putativeLamports) {
  return putativeLamports >= 0 && putativeLamports <= maxU64Value;
}
function assertIsLamports(putativeLamports) {
  if (putativeLamports < 0) {
    throw new Error("Input for 64-bit unsigned integer cannot be negative");
  }
  if (putativeLamports > maxU64Value) {
    throw new Error("Input number is too large to be represented as a 64-bit unsigned integer");
  }
}
function lamports(putativeLamports) {
  assertIsLamports(putativeLamports);
  return putativeLamports;
}

// src/params-patcher.ts
function visitNode(value, keyPath, onIntegerOverflow) {
  if (Array.isArray(value)) {
    return value.map(
      (element, ii) => visitNode(element, [...keyPath, ii], onIntegerOverflow)
    );
  } else if (typeof value === "object" && value !== null) {
    const out = {};
    for (const propName in value) {
      if (Object.prototype.hasOwnProperty.call(value, propName)) {
        out[propName] = visitNode(value[propName], [...keyPath, propName], onIntegerOverflow);
      }
    }
    return out;
  } else if (typeof value === "bigint") {
    if (onIntegerOverflow && (value > Number.MAX_SAFE_INTEGER || value < -Number.MAX_SAFE_INTEGER)) {
      onIntegerOverflow(keyPath, value);
    }
    return Number(value);
  } else {
    return value;
  }
}
function patchParamsForSolanaLabsRpc(params, onIntegerOverflow) {
  return visitNode(params, [], onIntegerOverflow);
}

// src/response-patcher-types.ts
var KEYPATH_WILDCARD = {};

// src/response-patcher-allowed-numeric-values.ts
var memoizedNotificationKeypaths;
var memoizedResponseKeypaths;
function getAllowedNumericKeypathsForNotification() {
  if (!memoizedNotificationKeypaths) {
    memoizedNotificationKeypaths = {};
  }
  return memoizedNotificationKeypaths;
}
function getAllowedNumericKeypathsForResponse() {
  if (!memoizedResponseKeypaths) {
    const jsonParsedTokenAccountsConfigs = [
      // parsed Token/Token22 token account
      ["data", "parsed", "info", "tokenAmount", "decimals"],
      ["data", "parsed", "info", "tokenAmount", "uiAmount"],
      ["data", "parsed", "info", "rentExemptReserve", "decimals"],
      ["data", "parsed", "info", "rentExemptReserve", "uiAmount"],
      ["data", "parsed", "info", "delegatedAmount", "decimals"],
      ["data", "parsed", "info", "delegatedAmount", "uiAmount"],
      [
        "data",
        "parsed",
        "info",
        "extensions",
        KEYPATH_WILDCARD,
        "state",
        "olderTransferFee",
        "transferFeeBasisPoints"
      ],
      [
        "data",
        "parsed",
        "info",
        "extensions",
        KEYPATH_WILDCARD,
        "state",
        "newerTransferFee",
        "transferFeeBasisPoints"
      ],
      ["data", "parsed", "info", "extensions", KEYPATH_WILDCARD, "state", "preUpdateAverageRate"],
      ["data", "parsed", "info", "extensions", KEYPATH_WILDCARD, "state", "currentRate"]
    ];
    const jsonParsedAccountsConfigs = [
      ...jsonParsedTokenAccountsConfigs,
      // parsed AddressTableLookup account
      ["data", "parsed", "info", "lastExtendedSlotStartIndex"],
      // parsed Config account
      ["data", "parsed", "info", "slashPenalty"],
      ["data", "parsed", "info", "warmupCooldownRate"],
      // parsed Token/Token22 mint account
      ["data", "parsed", "info", "decimals"],
      // parsed Token/Token22 multisig account
      ["data", "parsed", "info", "numRequiredSigners"],
      ["data", "parsed", "info", "numValidSigners"],
      // parsed Stake account
      ["data", "parsed", "info", "stake", "delegation", "warmupCooldownRate"],
      // parsed Sysvar rent account
      ["data", "parsed", "info", "exemptionThreshold"],
      ["data", "parsed", "info", "burnPercent"],
      // parsed Vote account
      ["data", "parsed", "info", "commission"],
      ["data", "parsed", "info", "votes", KEYPATH_WILDCARD, "confirmationCount"]
    ];
    memoizedResponseKeypaths = {
      getAccountInfo: jsonParsedAccountsConfigs.map((c) => ["value", ...c]),
      getBlock: [
        ["blockTime"],
        ["transactions", KEYPATH_WILDCARD, "meta", "preTokenBalances", KEYPATH_WILDCARD, "accountIndex"],
        [
          "transactions",
          KEYPATH_WILDCARD,
          "meta",
          "preTokenBalances",
          KEYPATH_WILDCARD,
          "uiTokenAmount",
          "decimals"
        ],
        ["transactions", KEYPATH_WILDCARD, "meta", "postTokenBalances", KEYPATH_WILDCARD, "accountIndex"],
        [
          "transactions",
          KEYPATH_WILDCARD,
          "meta",
          "postTokenBalances",
          KEYPATH_WILDCARD,
          "uiTokenAmount",
          "decimals"
        ],
        ["transactions", KEYPATH_WILDCARD, "meta", "rewards", KEYPATH_WILDCARD, "commission"],
        ["transactions", KEYPATH_WILDCARD, "meta", "innerInstructions", KEYPATH_WILDCARD, "index"],
        [
          "transactions",
          KEYPATH_WILDCARD,
          "meta",
          "innerInstructions",
          KEYPATH_WILDCARD,
          "instructions",
          KEYPATH_WILDCARD,
          "programIdIndex"
        ],
        [
          "transactions",
          KEYPATH_WILDCARD,
          "meta",
          "innerInstructions",
          KEYPATH_WILDCARD,
          "instructions",
          KEYPATH_WILDCARD,
          "accounts",
          KEYPATH_WILDCARD
        ],
        [
          "transactions",
          KEYPATH_WILDCARD,
          "transaction",
          "message",
          "addressTableLookups",
          KEYPATH_WILDCARD,
          "writableIndexes",
          KEYPATH_WILDCARD
        ],
        [
          "transactions",
          KEYPATH_WILDCARD,
          "transaction",
          "message",
          "addressTableLookups",
          KEYPATH_WILDCARD,
          "readonlyIndexes",
          KEYPATH_WILDCARD
        ],
        [
          "transactions",
          KEYPATH_WILDCARD,
          "transaction",
          "message",
          "instructions",
          KEYPATH_WILDCARD,
          "programIdIndex"
        ],
        [
          "transactions",
          KEYPATH_WILDCARD,
          "transaction",
          "message",
          "instructions",
          KEYPATH_WILDCARD,
          "accounts",
          KEYPATH_WILDCARD
        ],
        ["transactions", KEYPATH_WILDCARD, "transaction", "message", "header", "numReadonlySignedAccounts"],
        ["transactions", KEYPATH_WILDCARD, "transaction", "message", "header", "numReadonlyUnsignedAccounts"],
        ["transactions", KEYPATH_WILDCARD, "transaction", "message", "header", "numRequiredSignatures"],
        ["rewards", KEYPATH_WILDCARD, "commission"]
      ],
      getBlockTime: [[]],
      getClusterNodes: [
        [KEYPATH_WILDCARD, "featureSet"],
        [KEYPATH_WILDCARD, "shredVersion"]
      ],
      getInflationGovernor: [["initial"], ["foundation"], ["foundationTerm"], ["taper"], ["terminal"]],
      getInflationRate: [["foundation"], ["total"], ["validator"]],
      getInflationReward: [[KEYPATH_WILDCARD, "commission"]],
      getMultipleAccounts: jsonParsedAccountsConfigs.map((c) => ["value", KEYPATH_WILDCARD, ...c]),
      getProgramAccounts: jsonParsedAccountsConfigs.flatMap((c) => [
        ["value", KEYPATH_WILDCARD, "account", ...c],
        [KEYPATH_WILDCARD, "account", ...c]
      ]),
      getRecentPerformanceSamples: [[KEYPATH_WILDCARD, "samplePeriodSecs"]],
      getTokenAccountBalance: [
        ["value", "decimals"],
        ["value", "uiAmount"]
      ],
      getTokenAccountsByDelegate: jsonParsedTokenAccountsConfigs.map((c) => [
        "value",
        KEYPATH_WILDCARD,
        "account",
        ...c
      ]),
      getTokenAccountsByOwner: jsonParsedTokenAccountsConfigs.map((c) => [
        "value",
        KEYPATH_WILDCARD,
        "account",
        ...c
      ]),
      getTokenLargestAccounts: [
        ["value", KEYPATH_WILDCARD, "decimals"],
        ["value", KEYPATH_WILDCARD, "uiAmount"]
      ],
      getTokenSupply: [
        ["value", "decimals"],
        ["value", "uiAmount"]
      ],
      getTransaction: [
        ["meta", "preTokenBalances", KEYPATH_WILDCARD, "accountIndex"],
        ["meta", "preTokenBalances", KEYPATH_WILDCARD, "uiTokenAmount", "decimals"],
        ["meta", "postTokenBalances", KEYPATH_WILDCARD, "accountIndex"],
        ["meta", "postTokenBalances", KEYPATH_WILDCARD, "uiTokenAmount", "decimals"],
        ["meta", "rewards", KEYPATH_WILDCARD, "commission"],
        ["meta", "innerInstructions", KEYPATH_WILDCARD, "index"],
        ["meta", "innerInstructions", KEYPATH_WILDCARD, "instructions", KEYPATH_WILDCARD, "programIdIndex"],
        [
          "meta",
          "innerInstructions",
          KEYPATH_WILDCARD,
          "instructions",
          KEYPATH_WILDCARD,
          "accounts",
          KEYPATH_WILDCARD
        ],
        [
          "transaction",
          "message",
          "addressTableLookups",
          KEYPATH_WILDCARD,
          "writableIndexes",
          KEYPATH_WILDCARD
        ],
        [
          "transaction",
          "message",
          "addressTableLookups",
          KEYPATH_WILDCARD,
          "readonlyIndexes",
          KEYPATH_WILDCARD
        ],
        ["transaction", "message", "instructions", KEYPATH_WILDCARD, "programIdIndex"],
        ["transaction", "message", "instructions", KEYPATH_WILDCARD, "accounts", KEYPATH_WILDCARD],
        ["transaction", "message", "header", "numReadonlySignedAccounts"],
        ["transaction", "message", "header", "numReadonlyUnsignedAccounts"],
        ["transaction", "message", "header", "numRequiredSignatures"]
      ],
      getVersion: [["feature-set"]],
      getVoteAccounts: [
        ["current", KEYPATH_WILDCARD, "commission"],
        ["delinquent", KEYPATH_WILDCARD, "commission"]
      ],
      simulateTransaction: jsonParsedAccountsConfigs.map((c) => ["value", "accounts", KEYPATH_WILDCARD, ...c])
    };
  }
  return memoizedResponseKeypaths;
}

// src/response-patcher.ts
function getNextAllowedKeypaths(keyPaths, property) {
  return keyPaths.filter((keyPath) => keyPath[0] === KEYPATH_WILDCARD && typeof property === "number" || keyPath[0] === property).map((keyPath) => keyPath.slice(1));
}
function visitNode2(value, allowedKeypaths) {
  if (Array.isArray(value)) {
    return value.map((element, ii) => {
      const nextAllowedKeypaths = getNextAllowedKeypaths(allowedKeypaths, ii);
      return visitNode2(element, nextAllowedKeypaths);
    });
  } else if (typeof value === "object" && value !== null) {
    const out = {};
    for (const [propName, innerValue] of Object.entries(value)) {
      const nextAllowedKeypaths = getNextAllowedKeypaths(allowedKeypaths, propName);
      out[propName] = visitNode2(innerValue, nextAllowedKeypaths);
    }
    return out;
  } else if (typeof value === "number" && // The presence of an allowed keypath on the route to this value implies it's allowlisted;
  // Upcast the value to `bigint` unless an allowed keypath is present.
  allowedKeypaths.length === 0 && // Only try to upcast an Integer to `bigint`
  Number.isInteger(value)) {
    return BigInt(value);
  } else {
    return value;
  }
}
function patchResponseForSolanaLabsRpc(rawResponse, methodName) {
  const allowedKeypaths = methodName ? getAllowedNumericKeypathsForResponse()[methodName] : void 0;
  return visitNode2(rawResponse, allowedKeypaths ?? []);
}
function patchResponseForSolanaLabsRpcSubscriptions(rawResponse, methodName) {
  const allowedKeypaths = methodName ? getAllowedNumericKeypathsForNotification()[methodName] : void 0;
  return visitNode2(rawResponse, allowedKeypaths ?? []);
}

// src/rpc-methods/index.ts
function createSolanaRpcApi(config) {
  return new Proxy({}, {
    defineProperty() {
      return false;
    },
    deleteProperty() {
      return false;
    },
    get(...args) {
      const [_, p] = args;
      const methodName = p.toString();
      return function(...rawParams) {
        const handleIntegerOverflow = config?.onIntegerOverflow;
        const params = patchParamsForSolanaLabsRpc(
          rawParams,
          handleIntegerOverflow ? (keyPath, value) => handleIntegerOverflow(methodName, keyPath, value) : void 0
        );
        return {
          methodName,
          params,
          responseProcessor: (rawResponse) => patchResponseForSolanaLabsRpc(rawResponse, methodName)
        };
      };
    }
  });
}

// src/rpc-subscriptions/index.ts
function createSolanaRpcSubscriptionsApi(config) {
  return new Proxy({}, {
    defineProperty() {
      return false;
    },
    deleteProperty() {
      return false;
    },
    get(...args) {
      const [_, p] = args;
      const notificationName = p.toString();
      return function(...rawParams) {
        const handleIntegerOverflow = config?.onIntegerOverflow;
        const params = patchParamsForSolanaLabsRpc(
          rawParams,
          handleIntegerOverflow ? (keyPath, value) => handleIntegerOverflow(notificationName, keyPath, value) : void 0
        );
        return {
          params,
          responseProcessor: (rawResponse) => patchResponseForSolanaLabsRpcSubscriptions(rawResponse, notificationName),
          subscribeMethodName: notificationName.replace(/Notifications$/, "Subscribe"),
          unsubscribeMethodName: notificationName.replace(/Notifications$/, "Unsubscribe")
        };
      };
    }
  });
}

// src/stringified-bigint.ts
function isStringifiedBigInt(putativeBigInt) {
  try {
    BigInt(putativeBigInt);
    return true;
  } catch (_) {
    return false;
  }
}
function assertIsStringifiedBigInt(putativeBigInt) {
  try {
    BigInt(putativeBigInt);
  } catch (e) {
    throw new Error(`\`${putativeBigInt}\` cannot be parsed as a BigInt`, {
      cause: e
    });
  }
}
function stringifiedBigInt(putativeBigInt) {
  assertIsStringifiedBigInt(putativeBigInt);
  return putativeBigInt;
}
function isTransactionSignature(putativeTransactionSignature) {
  if (
    // Lowest value (64 bytes of zeroes)
    putativeTransactionSignature.length < 64 || // Highest value (64 bytes of 255)
    putativeTransactionSignature.length > 88
  ) {
    return false;
  }
  const bytes = umiSerializers.base58.serialize(putativeTransactionSignature);
  const numBytes = bytes.byteLength;
  if (numBytes !== 64) {
    return false;
  }
  return true;
}
function assertIsTransactionSignature(putativeTransactionSignature) {
  try {
    if (
      // Lowest value (64 bytes of zeroes)
      putativeTransactionSignature.length < 64 || // Highest value (64 bytes of 255)
      putativeTransactionSignature.length > 88
    ) {
      throw new Error("Expected input string to decode to a byte array of length 64.");
    }
    const bytes = umiSerializers.base58.serialize(putativeTransactionSignature);
    const numBytes = bytes.byteLength;
    if (numBytes !== 64) {
      throw new Error(`Expected input string to decode to a byte array of length 64. Actual length: ${numBytes}`);
    }
  } catch (e) {
    throw new Error(`\`${putativeTransactionSignature}\` is not a transaction signature`, {
      cause: e
    });
  }
}
function transactionSignature(putativeTransactionSignature) {
  assertIsTransactionSignature(putativeTransactionSignature);
  return putativeTransactionSignature;
}

// src/unix-timestamp.ts
function isUnixTimestamp(putativeTimestamp) {
  if (putativeTimestamp > 864e13 || putativeTimestamp < -864e13) {
    return false;
  }
  return true;
}
function assertIsUnixTimestamp(putativeTimestamp) {
  try {
    if (putativeTimestamp > 864e13 || putativeTimestamp < -864e13) {
      throw new Error("Expected input number to be in the range [-8.64e15, 8.64e15]");
    }
  } catch (e) {
    throw new Error(`\`${putativeTimestamp}\` is not a timestamp`, {
      cause: e
    });
  }
}
function unixTimestamp(putativeTimestamp) {
  assertIsUnixTimestamp(putativeTimestamp);
  return putativeTimestamp;
}

exports.assertIsLamports = assertIsLamports;
exports.assertIsStringifiedBigInt = assertIsStringifiedBigInt;
exports.assertIsTransactionSignature = assertIsTransactionSignature;
exports.assertIsUnixTimestamp = assertIsUnixTimestamp;
exports.createSolanaRpcApi = createSolanaRpcApi;
exports.createSolanaRpcSubscriptionsApi = createSolanaRpcSubscriptionsApi;
exports.isLamports = isLamports;
exports.isStringifiedBigInt = isStringifiedBigInt;
exports.isTransactionSignature = isTransactionSignature;
exports.isUnixTimestamp = isUnixTimestamp;
exports.lamports = lamports;
exports.stringifiedBigInt = stringifiedBigInt;
exports.transactionSignature = transactionSignature;
exports.unixTimestamp = unixTimestamp;
//# sourceMappingURL=out.js.map
//# sourceMappingURL=index.node.cjs.map